@@ -1190,12 +1190,16 @@ else {
 	}
 }
 
-# make_chroot(file)
+# make_chroot(file, [is-pid])
 sub make_chroot
 {
 local $chroot = &get_chroot();
 return $_[0] if (!$chroot);
 return $_[0] if ($_[0] eq $config{'named_conf'} && $config{'no_chroot'});
+if ($config{'no_pid_chroot'} &&
+    ($_[1] || $_[0] eq &get_pid_file())) {
+	return $_[0];
+	}
 return $chroot.$_[0];
 }
 
@@ -1213,7 +1217,7 @@ return 0;
 }
 
 # get_pid_file()
-# Returns the BIND pid file path
+# Returns the BIND pid file path, relative to any chroot
 sub get_pid_file
 {
 local $conf = &get_config();
@@ -1231,7 +1235,7 @@ if (($opts = &find("options", $conf)) &&
 
 # use default file
 foreach $p (split(/\s+/, $config{'pid_file'})) {
-	if (-r &make_chroot($p)) {
+	if (-r &make_chroot($p, 1)) {
 		return $p;
 		}
 	}
@@ -1364,7 +1368,7 @@ return 0;
 sub restart_bind
 {
 if ($config{'restart_cmd'}) {
-	local $out = &backquote_logged("$config{'restart_cmd'}2>&1 </dev/null");
+	local $out = &backquote_logged("$config{'restart_cmd'} 2>&1 </dev/null");
 	if ($?) {
 		return &text('restart_ecmd', "<pre>$out</pre>");
 		}
@@ -1372,10 +1376,13 @@ if ($config{'restart_cmd'}) {
 else {
 	local $pidfile = &get_pid_file();
 	local $pid;
-	open(PIDFILE, $pidfile);
+	open(PIDFILE, &make_chroot($pidfile));
 	chop($pid = <PIDFILE>);
 	close(PIDFILE);
-	if (!$pid || !&kill_logged('HUP', $pid)) {
+	if (!$pid) {
+		return &text('restart_epidfile', $pidfile);
+		}
+	elsif (!&kill_logged('HUP', $pid)) {
 		return &text('restart_esig', $pid, $!);
 		}
 	}
@@ -1394,7 +1401,21 @@ for($i=0; $i<@vsp || $i<@_; $i++) {
 return 1;	# same!
 }
 
-
+# get_zone_index(name, [view])
+# Returns the index of some zone in the real on-disk configuration
+sub get_zone_index
+{
+undef(@get_config_cache);
+local $conf = &get_config();
+local $vconf = $_[1] ne '' ? $conf->[$in{'view'}]->{'members'} : $conf;
+local $c;
+foreach $c (@$vconf) {
+	if ($c->{'name'} eq 'zone' && $c->{'value'} eq $_[0]) {
+		return $c->{'index'};
+		}
+	}
+return undef;
+}
 
 1;
 
