@@ -16,8 +16,20 @@ if ($type eq 'record') {
 		}
 	else {
 		$p->{'name'} =~ s/\.$object\.*$//;
-		return &text("log_${action}_record", $text{"type_$p->{'type'}"},
-			     "<tt>$p->{'name'}</tt>", "<tt>$object</tt>");
+		if (($action eq "modify" || $action eq "create") &&
+		    $p->{'newvalues'}) {
+			return &text("log_${action}_record_v",
+				     $text{"type_$p->{'type'}"},
+				     "<tt>$p->{'name'}</tt>",
+				     "<tt>$object</tt>",
+				     "<tt>$p->{'newvalues'}</tt>");
+			}
+		else {
+			return &text("log_${action}_record",
+				     $text{"type_$p->{'type'}"},
+				     "<tt>$p->{'name'}</tt>",
+				     "<tt>$object</tt>");
+			}
 		}
 	}
 elsif ($action eq 'create') {
