@@ -2,9 +2,11 @@
 # Deal with debian's iptables save file and startup script
 
 $has_debian_iptables = -r "/etc/init.d/iptables";
+$debian_iptables_dir = "/var/lib/iptables";
 
 if ($has_debian_iptables) {
-	$iptables_save_file = "/var/lib/iptables/active";
+	mkdir($debian_iptables_dir, 0755) if (!-d $debian_iptables_dir);
+	$iptables_save_file = "$debian_iptables_dir/active";
 	}
 
 # apply_iptables()
