@@ -328,6 +328,8 @@ sub can_edit_table
 return $access{$_[0]};
 }
 
+# run_before_command()
+# Runs the before-saving command, if any
 sub run_before_command
 {
 if ($config{'before_cmd'}) {
@@ -335,6 +337,8 @@ if ($config{'before_cmd'}) {
 	}
 }
 
+# run_after_command()
+# Runs the after-saving command, if any
 sub run_after_command
 {
 if ($config{'after_cmd'}) {
@@ -342,5 +346,130 @@ if ($config{'after_cmd'}) {
 	}
 }
 
+# run_before_apply_command()
+# Runs the before-applying command, if any. If it failes, returns the error
+# message output
+sub run_before_apply_command
+{
+if ($config{'before_apply_cmd'}) {
+	local $out = &backquote_logged("($config{'before_apply_cmd'}) </dev/null 2>&1");
+	return $out if ($?);
+	}
+return undef;
+}
+
+# run_after_apply_command()
+# Runs the after-applying command, if any
+sub run_after_apply_command
+{
+if ($config{'after_apply_cmd'}) {
+	&system_logged("($config{'after_apply_cmd'}) </dev/null >/dev/null 2>&1");
+	}
+}
+
+# apply_configuration()
+# Calls all the appropriate apply functions and programs, and returns an error
+# message if anything fails
+sub apply_configuration
+{
+local $err = &run_before_apply_command();
+return $err if ($err);
+if (defined(&apply_iptables)) {
+	# Call distro's apply command
+	$err = &apply_iptables();
+	}
+else {
+	# Manually run iptables-restore
+	$err = &iptables_restore();
+	}
+return $err if ($err);
+&run_after_apply_command();
+return undef;
+}
+
+# list_cluster_servers()
+# Returns a list of servers on which the firewall is managed
+sub list_cluster_servers
+{
+&foreign_require("servers", "servers-lib.pl");
+local %ids = map { $_, 1 } split(/\s+/, $config{'servers'});
+return grep { $ids{$_->{'id'}} } &servers::list_servers();
+}
+
+# add_cluster_server(&server)
+sub add_cluster_server
+{
+local @sids = split(/\s+/, $config{'servers'});
+$config{'servers'} = join(" ", @sids, $_[0]->{'id'});
+&save_module_config();
+}
+
+# delete_cluster_server(&server)
+sub delete_cluster_server
+{
+local @sids = split(/\s+/, $config{'servers'});
+$config{'servers'} = join(" ", grep { $_ != $_[0]->{'id'} } @sids);
+&save_module_config();
+}
+
+# server_name(&server)
+sub server_name
+{
+return $_[0]->{'desc'} ? $_[0]->{'desc'} : $_[0]->{'host'};
+}
+
+# copy_to_cluster([force])
+# Copy all firewall rules from this server to those in the cluster
+sub copy_to_cluster
+{
+return if (!$config{'servers'});		# no servers defined
+return if (!$_[0] && $config{'cluster_mode'});	# only push out when applying
+local $s;
+local $ltemp;
+if ($config{'direct'}) {
+	# Dump current configuration
+	$ltemp = &tempname();
+	system("iptables-save >$ltemp 2>/dev/null");
+	}
+foreach $s (&list_cluster_servers()) {
+	&remote_foreign_require($s->{'host'}, "firewall", "firewall-lib.pl");
+	if ($config{'direct'}) {
+		# Directly activate on remote server!
+		local $rtemp = &remote_write($s->{'host'}, $ltemp);
+		unlink($ltemp);
+		local $err = &remote_eval($s->{'host'}, "firewall",
+		  "\$out = `iptables-restore <$rtemp 2>&1`; [ \$out, \$? ]"); 
+		&remote_eval($s->{'host'}, "firewall", "unlink('$rtemp')");
+		&error(&text('apply_remote', $s->{'host'}, $err->[0]))
+			if ($err->[1]);
+		}
+	else {
+		# Can just copy across save file
+		local $rfile = &remote_eval($s->{'host'}, "firewall",
+					    "\$iptables_save_file");
+		&remote_write($s->{'host'}, $iptables_save_file, $rfile);
+		}
+	}
+}
+
+# apply_cluster_configuration()
+# Activate the current configuration on all servers in the cluster
+sub apply_cluster_configuration
+{
+return undef if (!$config{'servers'});
+if ($config{'cluster_mode'}) {
+	&copy_to_cluster(1);
+	}
+local $s;
+foreach $s (&list_cluster_servers()) {
+	&remote_foreign_require($s->{'host'}, "firewall", "firewall-lib.pl");
+	local $err = &remote_foreign_call($s->{'host'}, "firewall", "apply_configuration");
+	if ($err) {
+		return &text('apply_remote', $s->{'host'}, $err);
+		}
+	}
+return undef;
+}
+
 1;
 
