@@ -53,6 +53,7 @@ else {
 			}
 		elsif (/VG\s+Name\s+(.*)/i) {
 			$pv->{'vg'} = $1;
+			$pv->{'vg'} =~ s/\s+\(.*\)//;
 			}
 		elsif (/PV\s+Size\s+(\S+)\s+(\S+)/i) {
 			$pv->{'size'} = &mult_units($1, $2);
@@ -486,6 +487,8 @@ elsif ($has_raid) {
 return ();
 }
 
+# device_message(stat)
+# Returns a text string about the status of an LV
 sub device_message
 {
 $msg = $_[2] ? 'lv_mount' : 'lv_umount';
