@@ -8,7 +8,7 @@ use POSIX;
 sub list_mails
 {
 local (@rv, $h, $done);
-local (@index, %index, $itype);
+my (@index, %index, $itype);
 $itype = &index_type($_[0]);
 if ($itype == 0) {
 	@index = &build_index($_[0]);
@@ -56,7 +56,7 @@ for($i=$start; $i<=$end; $i++) {
 			push(@headers, [ $1, $2 ]);
 			$mail->{'rawheaders'} .= $line."\n";
 			}
-		elsif ($line =~ /^(\s+.*)/) {
+		elsif ($line =~ /^\s+(.*)/) {
 			$headers[$#headers]->[1] .= $1 unless($#headers < 0);
 			$mail->{'rawheaders'} .= $line."\n";
 			}
@@ -93,7 +93,7 @@ sub search_mail
 return &advanced_search_mail($_[0], [ [ $_[1], $_[2] ] ], 1);
 }
 
-# advanced_search_mail(user|file, &fields, andmode)
+# advanced_search_mail(user|file, &fields, andmode, [&limits])
 # Returns an array of messages matching some search
 sub advanced_search_mail
 {
@@ -102,10 +102,16 @@ local (@index, %index, @rv, $i);
 local $dash = &dash_mode($_[0]);
 local @possible;		# index positions of possible mails
 local $possible_certain = 0;	# is possible list authoratative?
+local ($min, $max);
 if ($itype == 0) {
 	# We have only a plain index ..
 	@index = &build_index($_[0]);
-	@possible = (0 .. scalar(@index)-1);
+	$min = 0;
+	$max = scalar(@index)-1;
+	if ($_[3] && $_[3]->{'latest'}) {
+		$min = $max - $_[3]->{'latest'};
+		}
+	@possible = ($min .. $max);
 	}
 else {
 	# We have a DBM index .. if the search includes the from and subject
@@ -117,11 +123,17 @@ else {
 				  $_->[0] eq 'subject' } @{$_[1]};
 	local $alldbm = (scalar(@dbmfields) == scalar(@{$_[1]}));
 
+	$min = 0;
+	$max = $index{'mailcount'}-1;
+	if ($_[3] && $_[3]->{'latest'}) {
+		$min = $max - $_[3]->{'latest'};
+		}
+
 	# Only check DBM if it contains some fields, and if it contains all
 	# fields when in 'or' mode.
 	if (@dbmfields && ($alldbm || $_[2])) {
 		# Scan the DBM to build up a list of 'possibles'
-		for($i=0; $i<$index{'mailcount'}; $i++) {
+		for($i=$min; $i<=$max; $i++) {
 			local @idx = split(/\0/, $index{$i});
 			local $fake = { 'header' => { 'from', $idx[2],
 						      'subject', $idx[3] } };
@@ -132,7 +144,7 @@ else {
 		}
 	else {
 		# None of the DBM fields are in the search .. have to scan all
-		@possible = (0 .. $index{'mailcount'}-1);
+		@possible = ($min .. $max);
 		}
 	}
 
@@ -161,7 +173,7 @@ foreach $i (@possible) {
 			push(@headers, [ $1, $2 ]);
 			$mail->{'rawheaders'} .= $line."\n";
 			}
-		elsif ($line =~ /^(\s+.*)/) {
+		elsif ($line =~ /^\s+(.*)/) {
 			$headers[$#headers]->[1] .= $1 unless($#headers < 0);
 			$mail->{'rawheaders'} .= $line."\n";
 			}
@@ -325,11 +337,11 @@ if (!@st || $index->{'lastchange'} < $st[9] || $st[7] < $config{'index_min'}) {
 		elsif ($_ eq "\n" || $_ eq "\r\n") {
 			$doingheaders = 0;
 			}
-		elsif ($doingheaders && /^From:\s*(.{0,255})/) {
+		elsif ($doingheaders && /^From:\s*(.{0,255})/i) {
 			$nidx[2] = $1;
 			$index->{$istart-1} = join("\0", @nidx);
 			}
-		elsif ($doingheaders && /^Subject:\s*(.{0,255})/) {
+		elsif ($doingheaders && /^Subject:\s*(.{0,255})/i) {
 			$nidx[3] = $1;
 			$index->{$istart-1} = join("\0", @nidx);
 			}
@@ -377,6 +389,22 @@ else {
 	}
 }
 
+# count_mail(user|file)
+# Returns the number of messages in some mail file
+sub count_mail
+{
+my (@index, %index, $itype);
+$itype = &index_type($_[0]);
+if ($itype == 0) {
+	@index = &build_index($_[0]);
+	return scalar(@index);
+	}
+else {
+	&build_dbm_index($_[0], \%index);
+	return $index{'mailcount'};
+	}
+}
+
 # parse_mail(&mail, [&parent], [savebody])
 # Extracts the attachments from the mail body
 sub parse_mail
@@ -403,7 +431,7 @@ if ($ct =~ /multipart\/(\S+)/i && ($ct =~ /boundary="([^"]+)"/i ||
 			if ($lines[$l] =~ /^(\S+):\s*(.*)/) {
 				push(@headers, [ $1, $2 ]);
 				}
-			elsif ($lines[$l] =~ /^(\s+.*)/) {
+			elsif ($lines[$l] =~ /^\s+(.*)/) {
 				$headers[$#headers]->[1] .= $1
 					unless($#headers < 0);
 				}
@@ -471,7 +499,7 @@ if ($ct =~ /multipart\/(\S+)/i && ($ct =~ /boundary="([^"]+)"/i ||
 				local $/ = undef;
 				$attach->{'data'} = <HEXBIN>;
 				close(HEXBIN);
-				local $ct = &guess_type($attach->{'filename'});
+				local $ct = &guess_mime_type($attach->{'filename'});
 				$attach->{'type'} = $ct;
 				$attach->{'header'} = { 'content-type' => $ct };
 				$attach->{'headers'} = [ [ 'Content-Type', $ct ] ];
@@ -526,12 +554,12 @@ if ($ct =~ /multipart\/(\S+)/i && ($ct =~ /boundary="([^"]+)"/i ||
 						$data .= $_;
 						}
 					close(FILE);
-					local $ct = &guess_type($f);
+					local $ct = &guess_mime_type($f);
 					push(@attach,
 					  { 'type' => $ct,
 					    'idx' => scalar(@attach),
 					    'header' =>
-						{ 'content-ype' => $ct },
+						{ 'content-type' => $ct },
 					    'headers' =>
 						[ [ 'Content-Type', $ct ] ],
 					    'filename' => $f,
@@ -553,7 +581,7 @@ elsif ($_[0]->{'body'} =~ /begin\s+([0-7]+)\s+(.*)/i) {
 	local ($attach, $rest);
 	foreach $l (@lines) {
 		if ($l =~ /^begin\s+([0-7]+)\s+(.*)/i) {
-			$attach = { 'type' => &guess_type($2),
+			$attach = { 'type' => &guess_mime_type($2),
 				    'idx' => scalar(@{$_[0]->{'attach'}}),
 				    'parent' => $_[1],
 				    'filename' => $2 };
@@ -773,19 +801,22 @@ chmod($st[2], $f);
 
 }
 
-# send_mail(&mail, [file], [textonly], [nocr], [smtp-server], [smtp-user], [smtp-pass])
+# send_mail(&mail, [file], [textonly], [nocr], [smtp-server],
+#	    [smtp-user], [smtp-pass], [smtp-auth-mode],
+#	    [&notify-flags])
 # Send out some email message or append it to a file.
 # Returns the number of lines written.
 sub send_mail
 {
 local (%header, $h);
-local $eol = $_[3] ? "\n" : "\r\n";
 local $lnum = 0;
 local $sm = $_[4] || $config{'send_mode'};
+local $eol = $_[3] || !$sm ? "\n" : "\r\n";
 foreach $h (@{$_[0]->{'headers'}}) {
 	$header{lc($h->[0])} = $h->[1];
 	}
 local @from = &address_parts($header{'from'});
+local $esmtp = $_[8] ? 1 : 0;
 if ($_[1]) {
 	# Just append the email to a file using mbox format
 	local @tm = localtime(time());
@@ -801,22 +832,79 @@ elsif ($sm) {
 	# Connect to SMTP server
 	&open_socket($sm, 25, MAIL);
 	&smtp_command(MAIL);
-	&smtp_command(MAIL, "helo ".&get_system_hostname()."\r\n");
+	if ($esmtp) {
+		&smtp_command(MAIL, "ehlo ".&get_system_hostname()."\r\n");
+		}
+	else {
+		&smtp_command(MAIL, "helo ".&get_system_hostname()."\r\n");
+		}
 
-	# XXX do SMTP auth here?
-	# XXX get username and password from parameters, or from module config
+	# Get username and password from parameters, or from module config
 	local $user = $_[5] || $userconfig{'smtp_user'} || $config{'smtp_user'};
 	local $pass = $_[6] || $userconfig{'smtp_pass'} || $config{'smtp_pass'};
+	local $auth = $_[7] || $userconfig{'smtp_auth'} ||
+		      $config{'smtp_auth'} || "Cram-MD5";
 	if ($user) {
 		# Send authentication commands
-		# XXX format?
+		eval "use Authen::SASL";
+		if ($@) {
+			&error("Perl module <tt>Authen::SASL</tt> is needed for SMTP authentication");
+			}
+		my $sasl = Authen::SASL->new('mechanism' => uc($auth),
+					     'callback' => {
+						'authname' => $user,
+						'user' => $user,
+						'pass' => $pass } );
+		&error("Failed to create Authen::SASL object") if (!$sasl);
+		local $conn = $sasl->client_new("smtp", &get_system_hostname());
+		local $arv = &smtp_command(MAIL, "auth $auth\r\n", 1);
+		if ($arv =~ /^(334)\s+(.*)/) {
+			# Server says to go ahead
+			$extra = $2;
+			local $initial = $conn->client_start();
+			local $auth_ok;
+			if ($initial) {
+				local $enc = &encode_base64($initial);
+				$enc =~ s/\r|\n//g;
+				$arv = &smtp_command(MAIL, "$enc\r\n", 1);
+				if ($arv =~ /^(\d+)\s+(.*)/) {
+					if ($1 == 235) {
+						$auth_ok = 1;
+						}
+					else {
+						&error("Unknown SMTP authentication response : $arv");
+						}
+					}
+				$extra = $2;
+				}
+			while(!$auth_ok) {
+				local $message = &decode_base64($extra);
+				local $return = $conn->client_step($message);
+				local $enc = &encode_base64($return);
+				$enc =~ s/\r|\n//g;
+				$arv = &smtp_command(MAIL, "$enc\r\n", 1);
+				if ($arv =~ /^(\d+)\s+(.*)/) {
+					if ($1 == 235) {
+						$auth_ok = 1;
+						}
+					elsif ($1 == 535) {
+						&error("SMTP authentication failed : $arv");
+						}
+					$extra = $2;
+					}
+				else {
+					&error("Unknown SMTP authentication response : $arv");
+					}
+				}
+			}
 		}
 
 	&smtp_command(MAIL, "mail from: <$from[0]>\r\n");
+	local $notify = $_[8] ? " NOTIFY=".join(",", @{$_[8]}) : "";
 	local $u;
 	foreach $u (&address_parts($header{'to'}.",".$header{'cc'}.
 						 ",".$header{'bcc'})) {
-		&smtp_command(MAIL, "rcpt to: <$u>\r\n");
+		&smtp_command(MAIL, "rcpt to: <$u>$notify\r\n");
 		}
 	&smtp_command(MAIL, "data\r\n");
 	}
@@ -881,7 +969,9 @@ if (defined($_[0]->{'body'})) {
 	}
 elsif (!$_[2]) {
 	# Sending MIME-encoded email
-	$ctype =~ s/;.*$//;
+	if ($ctype !~ /multipart\/report/i) {
+		$ctype =~ s/;.*$//;
+		}
 	print MAIL "MIME-Version: 1.0",$eol;
 	local $bound = "bound".time();
 	print MAIL "Content-Type: $ctype; boundary=\"$bound\"",$eol;
@@ -971,27 +1061,6 @@ sub b64decode
     return $res;
 }
 
-sub guess_type
-{
-local $e;
-if (!%mime_types) {
-	open(MIME, "$root_directory/mime.types");
-	while(<MIME>) {
-		s/\r|\n//g;
-		s/#.*$//g;
-		local @s = split(/\s+/);
-		foreach $e (@s[1..$#s]) {
-			$mime_types{$e} = $s[0];
-			}
-		}
-	close(MIME);
-	}
-if ($_[0] =~ /\.([A-z0-9]+)$/ && $mime_types{lc($1)}) {
-	return $mime_types{lc($1)};
-	}
-return "application/octet-stream";
-}
-
 # can_read_mail(user)
 sub can_read_mail
 {
@@ -1058,7 +1127,7 @@ while(<QF>) {
 		push(@headers, [ $1, $2 ]);
 		$mail->{'rawheaders'} .= "$1: $2\n";
 		}
-	elsif (/^(\s+.*)/) {
+	elsif (/^\s+(.*)/) {
 		$headers[$#headers]->[1] .= $1 unless($#headers < 0);
 		$mail->{'rawheaders'} .= $_."\n";
 		}
@@ -1113,16 +1182,32 @@ foreach $rest (split(/\n/, $_[0])) {
 return @rv;
 }
 
-# smtp_command(handle, command)
+# smtp_command(handle, command, no-error)
 sub smtp_command
 {
 local ($m, $c) = @_;
 print $m $c;
 local $r = <$m>;
-if ($r !~ /^[23]\d+/) {
+if ($r !~ /^[23]\d+/ && !$_[2]) {
 	&error(&text('send_esmtp', "<tt>".&html_escape($c)."</tt>",
 				   "<tt>".&html_escape($r)."</tt>"));
 	}
+$r =~ s/\r|\n//g;
+if ($r =~ /^(\d+)\-/) {
+	# multi-line ESMTP response!
+	while(1) {
+		local $nr = <$m>;
+		$nr =~ s/\r|\n//g;
+		if ($nr =~ /^(\d+)\-(.*)/) {
+			$r .= "\n".$2;
+			}
+		elsif ($nr =~ /^(\d+)\s+(.*)/) {
+			$r .= "\n".$2;
+			last;
+			}
+		}
+	}
+return $r;
 }
 
 # address_parts(string)
@@ -1153,11 +1238,11 @@ local $l = $_[0];
 local $rv;
 while($l =~ /^(.*?)((http|ftp|https|mailto):[^><"'\s]+[^><"'\s\.\)])(.*)/) {
 	local ($before, $url, $after) = ($1, $2, $4);
-	$rv .= &eucconv(&html_escape($before))."<a href='$url'>".
+	$rv .= &eucconv_and_escape($before)."<a href='$url'>".
 	       &html_escape($url)."</a>";
 	$l = $after;
 	}
-$rv .= &eucconv(&html_escape($l));
+$rv .= &eucconv_and_escape($l);
 return $rv;
 }
 
@@ -1370,7 +1455,7 @@ while($i < @alines && $alines[$i]) {
 		push(@aheaders, [ $1, $2 ]);
 		$amail->{'rawheaders'} .= $alines[$i]."\n";
 		}
-	elsif ($alines[$i] =~ /^(\s+.*)/) {
+	elsif ($alines[$i] =~ /^\s+(.*)/) {
 		$aheaders[$#aheaders]->[1] .= $1 unless($#aheaders < 0);
 		$amail->{'rawheaders'} .= $alines[$i]."\n";
 		}
@@ -1399,6 +1484,7 @@ while(1) {
 		}
 	elsif ($str =~ /^[\s,]*("([^"]+)"\s*<([^\s<>,]+)>)(.*)$/ ||
 	       $str =~ /^[\s,]*(([^<>]+)\s+<([^\s<>,]+)>)(.*)$/ ||
+	       $str =~ /^[\s,]*(([^<>]+)<([^\s<>,]+)>)(.*)$/ ||
 	       $str =~ /^[\s,]*(([^<>\[\]]+)\s+\[mailto:([^\s\[\]]+)\])(.*)$/||
 	       $str =~ /^[\s,]*(()<([^<>,]+)>)(.*)/ ||
 	       $str =~ /^[\s,]*(()([^\s<>,]+))(.*)/) {
@@ -1436,6 +1522,11 @@ sub j2e {
 	$_;
 }
 
+# eucconv_and_escape(string)
+sub eucconv_and_escape {
+	return &html_escape(&eucconv($_[0]));
+}
+
 # list_maildir(file, [start], [end])
 # Returns a subset of mail from a maildir format directory
 sub list_maildir
@@ -1487,13 +1578,19 @@ sub search_maildir
 return &advanced_search_maildir($_[0], [ [ $_[1], $_[2] ] ], 1);
 }
 
-# advanced_search_maildir(user|file, &fields, andmode)
+# advanced_search_maildir(user|file, &fields, andmode, [&limit])
 # Search for messages in a maildir directory, and return the results
 sub advanced_search_maildir
 {
 local @rv;
-foreach $mail (&list_maildir($_[0])) {
-	push(@rv, $mail) if (&mail_matches($_[1], $_[2], $mail));
+local ($min, $max);
+if ($_[3] && $_[3]->{'latest'}) {
+	$min = -1;
+	$max = -$_[3]->{'latest'};
+	}
+foreach $mail (&list_maildir($_[0], $min, $max)) {
+	push(@rv, $mail) if ($mail &&
+			     &mail_matches($_[1], $_[2], $mail));
 	}
 return @rv;
 }
@@ -1547,6 +1644,21 @@ foreach $d ("$_[0]/cur", "$_[0]/new") {
 	}
 }
 
+# count_maildir(dir)
+# Returns the number of messages in a maildir directory
+sub count_maildir
+{
+local $d;
+local $count = 0;
+foreach $d ("$_[0]/cur", "$_[0]/new") {
+	opendir(DIR, $d);
+	local @files = grep { $_ !~ /^\./ } readdir(DIR);
+	$count += scalar(@files);
+	closedir(DIR);
+	}
+return $count;
+}
+
 # list_mhdir(file, [start], [end])
 # Returns a subset of mail from an MH format directory
 sub list_mhdir
@@ -1592,13 +1704,18 @@ sub search_mhdir
 return &advanced_search_mhdir($_[0], [ [ $_[1], $_[2] ] ], 1);
 }
 
-# advanced_search_mhdir(file|user, &fields, andmode)
+# advanced_search_mhdir(file|user, &fields, andmode, &limit)
 # Search for messages in an MH directory, and return the results
 sub advanced_search_mhdir
 {
 local @rv;
-foreach $mail (&list_mhdir($_[0])) {
-	push(@rv, $mail) if (&mail_matches($_[1], $_[2], $mail));
+local ($min, $max);
+if ($_[3] && $_[3]->{'latest'}) {
+	$min = -1;
+	$max = -$_[3]->{'latest'};
+	}
+foreach $mail (&list_mhdir($_[0], $min, $max)) {
+	push(@rv, $mail) if ($mail && &mail_matches($_[1], $_[2], $mail));
 	}
 return @rv;
 }
@@ -1646,6 +1763,16 @@ foreach $f (readdir(DIR)) {
 closedir(DIR);
 }
 
+# count_mhdir(file)
+# Returns the number of messages in an MH directory
+sub count_mhdir
+{
+opendir(DIR, $_[0]);
+local @files = grep { /^\d+$/ } readdir(DIR);
+closedir(DIR);
+return scalar(@files);
+}
+
 # read_mail_file(file, [headersonly])
 # Read a single message from a file
 sub read_mail_file
@@ -1663,7 +1790,7 @@ while(1) {
 		push(@headers, [ $1, $2 ]);
 		$mail->{'rawheaders'} .= $line."\n";
 		}
-	elsif ($line =~ /^(\s+.*)/) {
+	elsif ($line =~ /^\s+(.*)/) {
 		$headers[$#headers]->[1] .= $1 unless($#headers < 0);
 		$mail->{'rawheaders'} .= $line."\n";
 		}
@@ -1724,6 +1851,11 @@ foreach $f (@{$_[0]}) {
 		    if (!$neg && $headers =~ /\Q$what\E/i ||
 			 $neg && $headers !~ /\Q$what\E/i);
 		}
+	elsif ($field eq "status") {
+		$count++
+		    if (!$neg && $_[2]->{$field} =~ /\Q$what\E/i||
+		         $neg && $_[2]->{$field} !~ /\Q$what\E/i);
+		}
 	else {
 		$count++
 		    if (!$neg && $_[2]->{'header'}->{$field} =~ /\Q$what\E/i||
