@@ -454,18 +454,22 @@ sub regenerate_transport_table
 }
 
 
-# regenerate_any_table($parameter_where_to_find_the_table_names)
+# regenerate_any_table($parameter_where_to_find_the_table_names,
+#		       [ &force-files ])
 #
 sub regenerate_any_table
 {
-    if (&get_current_value($_[0]) ne "")
+    local @files;
+    if ($_[1]) {
+	@files = @{$_[1]};
+    } elsif (&get_current_value($_[0]) ne "") {
+	@files = &get_maps_files(&get_real_value($_[0]));
+    }
+    foreach $map (@files)
     {
-	foreach $map (&get_maps_files(&get_real_value($_[0])))
-	{
-	    next unless $map;
-	    $out = `$config{'postfix_lookup_table_command'} -c $config_dir $map 2>&1`;
-	    if ($out) { &error(&text('regenerate_table_efailed', $map, $out)); }
-	}
+        next unless $map;
+        local $out = `$config{'postfix_lookup_table_command'} -c $config_dir $map 2>&1`;
+        if ($out) { &error(&text('regenerate_table_efailed', $map, $out)); }
     }
 }
 
@@ -491,13 +495,13 @@ sub get_maps_files
 }
 
  
-# get_maps($maps_name) : \@maps
+# get_maps($maps_name, [&force-files]) : \@maps
 # construct the mappings database taken from the map files given from the parameter
 sub get_maps
 {
     if (!defined($maps_cache{$_[0]}))
     {
-	my @maps_files = &get_maps_files(&get_real_value($_[0]));
+	my @maps_files = $_[1] ? @{$_[1]} : &get_maps_files(&get_real_value($_[0]));
 	my $number = 0;
 	foreach $maps_file (@maps_files)
 	{
@@ -595,11 +599,11 @@ sub generate_map_edit
 }
 
 
-# create_mapping(map, &mapping)
+# create_mapping(map, &mapping, [&force-files])
 sub create_mapping
 {
-&get_maps($_[0]);	# force cache init
-my @maps_files = &get_maps_files(&get_real_value($_[0]));
+&get_maps($_[0], $_[2]);	# force cache init
+my @maps_files = $_[2] ? @{$_[2]} : &get_maps_files(&get_real_value($_[0]));
 local $lref = &read_file_lines($maps_files[0]);
 push(@$lref, "$_[1]->{'name'}\t$_[1]->{'value'}");
 &flush_file_lines();
@@ -699,7 +703,9 @@ sub parse_queue_file
 local @qfiles = ( &recurse_files("$config{'mailq_dir'}/active"),
 		  &recurse_files("$config{'mailq_dir'}/incoming"),
 		  &recurse_files("$config{'mailq_dir'}/deferred"),
-		  &recurse_files("$config{'mailq_dir'}/corrupt") );
+		  &recurse_files("$config{'mailq_dir'}/corrupt"),
+		  &recurse_files("$config{'mailq_dir'}/hold"),
+		);
 local $f = $_[0];
 local ($file) = grep { $_ =~ /\/$f$/ } @qfiles;
 return undef if (!$file);
