@@ -246,6 +246,9 @@ sub make_directives
 {
 local @rv;
 local $ver = $_[1];
+if ($ver =~ /^(1)\.(2)(\d+)$/) {
+	$ver = sprintf "%d.%d%2.2d", $1, $2, $3;
+	}
 foreach $d (@{$_[0]}) {
 	local(%dir);
 	$dir{'name'} = $d->[0];
@@ -723,7 +726,42 @@ if ($ENV{'QUERY_STRING'} ne "") { $url .= "?$ENV{'QUERY_STRING'}"; }
 return $url;
 }
 
-
+# running_under_inetd()
+# Returns the inetd/xinetd object and program if ProFTPd is running under one
+sub running_under_inetd
+{
+local ($inet, $inet_mod);
+if (&foreign_check('inetd')) {
+        # Check if proftpd is in inetd
+        &foreign_require('inetd', 'inetd-lib.pl');
+	local $i;
+        foreach $i (&foreign_call('inetd', 'list_inets')) {
+                if ($i->[1] && $i->[3] eq 'ftp') {
+                        $inet = $i;
+                        last;
+                        }
+                }
+        $inet_mod = 'inetd';
+        }
+elsif (&foreign_check('xinetd')) {
+        # Check if proftpd is in xinetd
+        &foreign_require('xinetd', 'xinetd-lib.pl');
+	local $xi;
+        foreach $xi (&foreign_call("xinetd", "get_xinetd_config")) {
+                if ($xi->{'quick'}->{'disable'}->[0] ne 'yes' &&
+                    $xi->{'value'} eq 'ftp') {
+                        $inet = $xi;
+                        last;
+                        }
+                }
+        $inet_mod = 'xinetd';
+        }
+else {
+        # Not supported on this OS .. assume so
+        $inet = 1;
+	}
+return ($inet, $inet_mod);
+}
 
 1;
 
