@@ -50,9 +50,24 @@ return $ver && $ver->{'value'} =~ /^(\d+)/ && $1 >= 7 ? $1 : undef;
 # Returns the actual sendmail executable version, if it is available
 sub get_sendmail_version
 {
-local $out = `$config{'sendmail_path'} -d0 -bv 2>&1 </dev/null`;
-${$_[0]} = $out;
-return $out =~ /version\s+(\S+)/i ? $1 : undef;
+local $version;
+local $out;
+open(OUT, "$config{'sendmail_path'} -d0 -bv 2>&1 </dev/null |");
+while(1) {
+	local $rmask;
+	vec($rmask, fileno(OUT), 1) = 1;
+	local $sel = select($rmask, undef, undef, 2);
+	last if (!$sel);
+	local $line = <OUT>;
+	last if (!$line);
+	if ($line =~ /version\s+(\S+)/i) {
+		$version = $1;
+		}
+	$out .= $line;
+	}
+close(OUT);
+${$_[0]} = $out if ($_[0]);
+return $version;
 }
 
 # save_directives(&config, &oldvalues, &newvalues)
