@@ -3,13 +3,20 @@
 
 do '../web-lib.pl';
 &init_config();
-require '../ui-lib.pl';
+do '../ui-lib.pl';
 
 # Get the version
 open(VERSION, "$module_config_directory/version");
 chop($shorewall_version = <VERSION>);
 close(VERSION);
 
+# get access permissions
+%access = &get_module_acl();
+
+@shorewall_files = ( 'zones', 'interfaces', 'policy', 'rules', 'tos',
+	   	     'masq', 'nat', 'proxyarp', 'routestopped',
+	   	     'tunnels', 'hosts', 'blacklist' );
+
 # version_atleast(v1, v2, v3)
 sub version_atleast
 {
@@ -37,6 +44,37 @@ close(FILE);
 return @rv;
 }
 
+# read_table_struct(table, &parserfunc)
+sub read_table_struct
+{
+if (!defined($read_table_cache{$_[0]})) {
+	local @rv;
+	local $func = $_[1];
+	open(FILE, "$config{'config_dir'}/$_[0]");
+	local $lnum = 0;
+	while(<FILE>) {
+		s/\r|\n//g;
+		local $cmt;
+		if (s/#\s*(.*)$//) {
+			$cmt = $1;
+			}
+		local $l = &$func($_);
+		if ($l) {
+			push(@rv, { 'line' => $lnum,
+				    'file' => "$config{'config_dir'}/$_[0]",
+				    'table' => $_[0],
+				    'index' => scalar(@rv),
+				    'values' => $l,
+				    'comment' => $cmt });
+			}
+		$lnum++;
+		}
+	close(FILE);
+	$read_table_cache{$_[0]} = \@rv;
+	}
+return $read_table_cache{$_[0]};
+}
+
 # find_line_num(&lref, &parserfunc, index)
 sub find_line_num
 {
@@ -62,6 +100,24 @@ splice(@$lref, $lnum, 1) if (defined($lnum));
 &flush_file_lines();
 }
 
+# delete_table_struct(&struct)
+sub delete_table_struct
+{
+local $lref = &read_file_lines($_[0]->{'file'});
+splice(@$lref, $_[0]->{'line'}, 1);
+&flush_file_lines();
+local $cache = $read_table_cache{$_[0]->{'table'}};
+local $idx = &indexof($_[0], @$cache);
+if ($idx >= 0) {
+	splice(@$cache, $idx, 1);
+	}
+local $c;
+foreach $c (@$cache) {
+	$c->{'line'}-- if ($c->{'line'} > $_[0]->{'line'});
+	$c->{'index'}-- if ($c->{'index'} > $_[0]->{'index'});
+	}
+}
+
 # create_table_row(table, &parserfunc, line, [insert-index])
 sub create_table_row
 {
@@ -84,6 +140,46 @@ else {
 &flush_file_lines();
 }
 
+# create_table_struct(&struct, parserfunc, [&insert-before])
+sub create_table_struct
+{
+local $lref = &read_file_lines("$config{'config_dir'}/$_[0]->{'table'}");
+local ($i, $idx);
+for($i=0; $i<@$lref; $i++) {
+	if ($lref->[$i] =~ /^#+\s*LAST\s+LINE/) {
+		$idx = $i;
+		last;
+		}
+	}
+if (!defined($idx)) {
+	$idx = @$lref;
+	}
+local $cache = &read_table_struct($_[0]->{'table'}, $_[1]);
+if ($_[2]) {
+	# Insert into file
+	splice(@$lref, $_[2]->{'line'}, 0, &make_struct($_[0]));
+	$_[0]->{'file'} = "$config{'config_dir'}/$_[0]->{'table'}";
+	$_[0]->{'line'} = $_[2]->{'line'};
+	$_[0]->{'index'} = $_[2]->{'index'};
+	local $c;
+	foreach $c (@$cache) {
+		$_[0]->{'line'}++ if ($c->{'line'} >= $_[2]->{'line'});
+		$_[0]->{'index'}++ if ($c->{'index'} >= $_[2]->{'index'});
+		}
+	local $iidx = &indexof($_[2], @$cache);
+	splice(@$cache, $iidx, 0, $_[0]);
+	}
+else {
+	# Append to file
+	splice(@$lref, $idx, 0, &make_struct($_[0]));
+	$_[0]->{'file'} = "$config{'config_dir'}/$_[0]->{'table'}";
+	$_[0]->{'line'} = $idx;
+	$_[0]->{'index'} = @$cache;
+	push(@$cache, $_[0]->{'index'});
+	}
+&flush_file_lines();
+}
+
 # modify_table_row(table, &parserfunc, index, line)
 sub modify_table_row
 {
@@ -93,6 +189,21 @@ $lref->[$lnum] = &simplify_line($_[3]) if (defined($lnum));
 &flush_file_lines();
 }
 
+# modify_table_struct(&newstruct, &oldstruct)
+sub modify_table_struct
+{
+local $lref = &read_file_lines("$config{'config_dir'}/$_[1]->{'table'}");
+$lref->[$_[1]->{'line'}] = &make_struct($_[0]);
+if ($_[0] ne $_[1]) {
+	$_[0]->{'line'} = $_[1]->{'line'};
+	$_[0]->{'index'} = $_[1]->{'index'};
+	local $cache = $read_table_cache{$_[1]->{'table'}};
+	local $idx = &indexof($_[1], @$cache);
+	$cache->[$idx] = $_[0];
+	}
+&flush_file_lines();
+}
+
 # swap_table_rows(table, &parserfunc, index1, index2)
 sub swap_table_rows
 {
@@ -103,6 +214,16 @@ local $lnum2 = &find_line_num($lref, $_[1], $_[3]);
 &flush_file_lines();
 }
 
+# make_struct(&struct)
+sub make_struct
+{
+local $line = join("\t", @{$_[0]->{'values'}});
+if ($_[0]->{'comment'}) {
+	$line .= "\t# $_[0]->{'comment'}";
+	}
+return &simplify_line($line);
+}
+
 # simplify_line(line)
 # Removes blank fields from the end of a line
 sub simplify_line
@@ -1108,7 +1229,17 @@ elsif ($in{'ports'}) {
 return ( $in{'host'}, $proto, $in{'ports'} );
 }
 
-
+# can_access(file)
+sub can_access
+{
+if ($access{'files'} eq '*') {
+	return 1;
+	}
+else {
+	local @acc = split(/\s+/, $access{'files'});
+	return &indexof($_[0], @acc) >= 0;
+	}
+}
 
 1;
 
