@@ -166,5 +166,16 @@ close(PROT);
 return &unique(@rv);
 }
 
+sub is_xinetd_running
+{
+if ($config{'pid_file'}) {
+	return &check_pid_file($config{'pid_file'});
+	}
+else {
+	local ($pid) = &find_byname("xinetd");
+	return $pid;
+	}
+}
+
 1;
 
